from pdb import set_trace

from django.contrib.syndication.feeds import FeedDoesNotExist, Feed
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
from django.shortcuts import get_object_or_404

from django_dzenlog.settings import HAS_TAGGING, RSS_LENGTH

from models import GeneralPost, published
from utils import upcast
from views import get_tagged, get_tags_by_slug

def latest(cls, list_url_name):
    class PostsFeed(Feed):
        title_template       = 'feeds/dzenlog_post_title.html'
        description_template = 'feeds/dzenlog_post_description.html'

        def __init__(self, *args, **kwargs):
            self.tags = None
            super(PostsFeed, self).__init__(*args, **kwargs)

        def title(self):
            params = {
                'site_name': Site.objects.get_current().name,
                'post_type': cls._meta.verbose_name_plural.lower(),
            }
            retval = _('%(site_name)s: latest %(post_type)s')

            if self.tags:
                if len(self.tags) == 1:
                    params['tag'] = u'"%s"' % self.tags[0]
                    retval = _('%(site_name)s: latest %(post_type)s with tag %(tag)s')
                else:
                    params['tags'] = u', '.join(u'"%s"' % tag for tag in self.tags)
                    retval = _('%(site_name)s: latest %(post_type)s with tags %(tags)s')
            return retval % params

        def get_object(self, bits):
            if len(bits) > 1:
                raise ObjectDoesNotExist
            if bits:
                self.tags = get_tags_by_slug(bits[0])
                return get_tagged(bits[0], published(cls.objects.all()), tags=self.tags)
            return None

        def categories(self, obj):
            if obj and len(obj) == 2:
                return obj[1]

        def link(self):
            return reverse(list_url_name)

        def items(self, obj):
            if obj:
                items = obj[0][:RSS_LENGTH]
            else:
                items = published(cls.objects.all())[:RSS_LENGTH]
            return [obj.upcast() for obj in items]

        def item_pubdate(self, item):
            return item.publish_at

        def item_author_name(self, item):
            return item.author

        def item_categories(self, item):
            if HAS_TAGGING:
                return item.get_tags()
    return PostsFeed

def latest_comments(cls, comments_mixup, details_page_name):
    class CommentsFeed(comments_mixup, Feed):
        title_template       = 'feeds/dzenlog_comment_title.html'
        description_template = 'feeds/dzenlog_comment_description.html'

        def __init__(self, *args, **kwargs):
            self.object = None
            super(CommentsFeed, self).__init__(*args, **kwargs)

        title = _('%s: comments') % Site.objects.get_current().name

        def get_object(self, bits):
            if len(bits) != 1:
                return None
            self.object = get_object_or_404(cls, slug=bits[0])
            self.title = _(u'%(site_name)s: comments on "%(title)s"') % {
                'title': self.object.title,
                'site_name': Site.objects.get_current().name
            }
            return self.object

        def link(self):
            if self.object is None:
                return ''
            return reverse(details_page_name, kwargs=dict(slug=self.object.slug))

    return CommentsFeed

LatestPosts = latest(GeneralPost, 'dzenlog-post-list')




from django.contrib import admin
from models import TextPost, LinkPost
from django_dzenlog.admin import GeneralPostAdmin

admin.site.register(TextPost, GeneralPostAdmin)
admin.site.register(LinkPost, GeneralPostAdmin)



from datetime import datetime, timedelta
from pdb import set_trace

from django.test import TestCase
from django.db import models
from django.dispatch import Signal
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse, NoReverseMatch
from django.template import Context
from django.template.loader import get_template_from_string

from tagging.models import Tag
from tagging.fields import TagField

from django_dzenlog.models import GeneralPost
from django_dzenlog.settings import HAS_TAGGING


class Parent(models.Model): pass
class Child(Parent): pass
class Child2(Parent): pass


class TestPost(GeneralPost): pass
class TestPost2(GeneralPost): pass

if HAS_TAGGING:
    class Tagging(TestCase):
        def setUp(self):
            (self.author, created) = User.objects.get_or_create(username='tester')

        def testSettingTags(self):
            p = TestPost(author=self.author, title='test', slug='test', tags='one, two, three')
            p.save()

            p2 = GeneralPost.objects.get(id=p.id)
            self.assertEqual('one, two, three', p2.tags)

            tags = Tag.objects.get_for_object(p2)
            self.assertEqual(3, len(tags))

        def testUpdateTags(self):
            p = TestPost(author=self.author, title='test', slug='test', tags='one, two, three')
            p.save()

            tags = p.get_tags()
            self.assertEqual(3, len(tags))

            p.tags = ''
            p.save()

            tags = p.get_tags()
            self.assertEqual(0, len(tags))

            p.tags = 'blah'
            p.save()

            tags = p.get_tags()
            self.assertEqual(1, len(tags))
            self.assertEqual('blah', tags[0].name)

from django.contrib.contenttypes.models import ContentType
from utils import upcast

class Upcast(TestCase):
    def setUp(self):
        Parent.objects.all().delete()

    def testUpcast(self):
        child = Child()
        child.save()

        parent = Parent.objects.get(id=child.parent_ptr_id)

        parent_ct = ContentType.objects.get_for_model(parent)
        parent_ct_upcasted = ContentType.objects.get_for_model(upcast(parent))
        child_ct = ContentType.objects.get_for_model(child)

        self.assertNotEqual(parent_ct, child_ct)
        self.assertEqual(parent_ct_upcasted, child_ct)
        self.assertEqual(upcast(child), child)

    def testUpcastWithMultipleChilds(self):
        child1 = Child()
        child1.save()

        child2 = Child2()
        child2.save()

        self.assertEqual(2, Parent.objects.all().count())

        parent1 = Parent.objects.get(id=child1.parent_ptr_id)
        parent1_ct = ContentType.objects.get_for_model(parent1)
        parent1_ct_upcasted = ContentType.objects.get_for_model(upcast(parent1))

        parent2 = Parent.objects.get(id=child2.parent_ptr_id)
        parent2_ct = ContentType.objects.get_for_model(parent2)
        parent2_ct_upcasted = ContentType.objects.get_for_model(upcast(parent2))

        child1_ct = ContentType.objects.get_for_model(child1)
        child2_ct = ContentType.objects.get_for_model(child2)

        self.assertNotEqual(child1_ct, child2_ct)

        self.assertNotEqual(parent1_ct, child1_ct)
        self.assertEqual(parent1_ct_upcasted, child1_ct)
        self.assertEqual(upcast(parent1), upcast(child1))

        self.assertNotEqual(parent2_ct, child2_ct)
        self.assertEqual(parent2_ct_upcasted, child2_ct)
        self.assertEqual(upcast(parent2), upcast(child2))

class PostsPublicity(TestCase):
    def setUp(self):
        (self.author, created) = User.objects.get_or_create(username='tester')
        TestPost.objects.all().delete()

        TestPost(author=self.author,
                 title='First post',
                 slug='first',
                 tags='one, two',
                 publish_at=datetime.today() - timedelta(0, 60)
                 ).save()
        TestPost(author=self.author,
                 title='Second post',
                 slug='second',
                 tags='one, two'
                 ).save()

    def testInHtml(self):
        response = self.client.get(reverse('dzenlog-generalpost-list'))
        self.assertContains(response, 'First post')
        self.assertNotContains(response, 'Second post')

    def testInRss(self):
        response = self.client.get(reverse('dzenlog-generalpost-feed', kwargs=dict(slug='rss', param='')))
        self.assertContains(response, 'First post')
        self.assertNotContains(response, 'Second post')

    if HAS_TAGGING:
        def testByTag(self):
            response = self.client.get(reverse('dzenlog-generalpost-bytag', kwargs=dict(slug='one')))
            self.assertContains(response, 'First post')
            self.assertNotContains(response, 'Second post')

        def testRssByTag(self):
            TestPost(author=self.author,
                     title='Third post',
                     slug='third',
                     tags='three, four',
                     publish_at=datetime.today() - timedelta(0, 60)
                     ).save()

            response = self.client.get(reverse('dzenlog-generalpost-bytag-feed', kwargs=dict(slug='rss', param='one')))
            self.assertContains(response, 'First post')
            self.assertNotContains(response, 'Second post')
            self.assertNotContains(response, 'Third post')

            response = self.client.get(reverse('dzenlog-generalpost-bytag-feed', kwargs=dict(slug='rss', param='three')))
            self.assertNotContains(response, 'First post')
            self.assertNotContains(response, 'Second post')
            self.assertContains(response, 'Third post')

    else:
        def testHasNoByTag(self):
            self.assertRaises(NoReverseMatch, reverse, 'dzenlog-generalpost-bytag', kwargs=dict(slug='one'))
            self.assertRaises(NoReverseMatch, reverse, 'dzenlog-generalpost-bytag-feed', kwargs=dict(slug='rss', param='one'))

class TemplateTags(TestCase):
    def testCall(self):
        t = get_template_from_string('''{% load dzenlog_tags %}{% call blah minor %}''')
        ctx = Context(dict(blah=lambda x: x, minor='test'))
        self.assertEqual('test', t.render(ctx))

    def testMethod(self):
        class Test:
            def blah(self, x):
                return x

        t = get_template_from_string('''{% load dzenlog_tags %}{% call obj.blah minor %}''')
        ctx = Context(dict(obj=Test(), minor='test'))
        self.assertEqual('test', t.render(ctx))

    def testFunctionNotFound(self):
        from django.template import TemplateSyntaxError
        t = get_template_from_string('''{% load dzenlog_tags %}{% call blah minor %}''')
        ctx = Context(dict(minor='test'))
        self.assertRaises(TemplateSyntaxError, t.render, ctx)

    def testObjectNotFound(self):
        from django.template import TemplateSyntaxError
        t = get_template_from_string('''{% load dzenlog_tags %}{% call obj.blah minor %}''')
        ctx = Context(dict(minor='test'))
        self.assertRaises(TemplateSyntaxError, t.render, ctx)

class Feeds(TestCase):
    def setUp(self):
        (self.author, created) = User.objects.get_or_create(username='tester')
        TestPost.objects.all().delete()

        TestPost(author=self.author,
                 title='First post',
                 slug='first',
                 tags='first-tag, second-tag',
                 publish_at=datetime.today() - timedelta(0, 60)
                 ).save()

    if HAS_TAGGING:
        def testPostCategoriesInRss(self):
            response = self.client.get(reverse('dzenlog-generalpost-feed', kwargs=dict(slug='rss', param='')))
            self.assertContains(response, 'first-tag')
            self.assertContains(response, 'second-tag')

            response = self.client.get(reverse('dzenlog-generalpost-bytag-feed', kwargs=dict(slug='rss', param='first-tag')))
            self.assertContains(response, 'first-tag')
            self.assertContains(response, 'second-tag')

        def testRequestedCategoriesInRss(self):
            TestPost(author=self.author,
                     title='Second post',
                     slug='second',
                     tags='another-tag',
                     ).save()

            tags = ['first-tag', 'another-tag']
            response = self.client.get(reverse('dzenlog-generalpost-bytag-feed', kwargs=dict(slug='rss', param='+'.join(tags))))
            for tag in tags:
                self.assertContains(response, tag)

if HAS_TAGGING:
    class ByTags(TestCase):
        def setUp(self):
            (self.author, created) = User.objects.get_or_create(username='tester')
            TestPost.objects.all().delete()

            TestPost(author=self.author,
                     title='First post',
                     slug='first',
                     tags='one, two',
                     publish_at=datetime.today() - timedelta(0, 60)
                     ).save()

            TestPost2(author=self.author,
                     title='Second post',
                     slug='second',
                     tags='three, four',
                     publish_at=datetime.today() - timedelta(0, 60)
                     ).save()

        def testPostCategoriesInRss(self):
            response = self.client.get(reverse('dzenlog-generalpost-tags'))
            for tag in ['one', 'two', 'three', 'four']:
                self.assertContains(response, tag)


class URLConf(TestCase):
    def testRaiseExceptionOnUnknownModel(self):
        from urls import create_patterns
        self.assertRaises(Exception, create_patterns, 'blah.minor')

class PublishedSignal(TestCase):
    def __init__(self, *args):
        (self.author, created) = User.objects.get_or_create(username='tester')
        self.sended = False

        def receiver(instance, sender, signal):
            self.assertEqual(TestPost, sender)
            self.sended = True

        from signals import published
        published.connect(receiver, sender = TestPost, weak = False)
        super(PublishedSignal, self).__init__(*args)

    def setUp(self):
        self.sended = False

    def testNotSendIfNoDate(self):
        TestPost(author=self.author,
                 title='Test post',
                 slug='first',
                 publish_at=None,
                 ).save()
        self.assertEqual(False, self.sended)

    def testSendIfDate(self):
        TestPost(author=self.author,
                 title='Test post',
                 slug='first',
                 publish_at=datetime.today(),
                 ).save()
        self.assertEqual(True, self.sended)

    def testNotSendOnRemoveFromPublication(self):
        post = TestPost(author=self.author,
                 title='Test post',
                 slug='first',
                 publish_at=datetime.today(),
                 )
        post.save()

        self.assertEqual(True, self.sended)
        self.sended = False

        post.publish_at = None
        post.save()
        self.assertEqual(False, self.sended)

    def testSendOnPublication(self):
        post = TestPost(author=self.author,
                 title='Test post',
                 slug='first',
                 publish_at=None,
                 )
        post.save()

        self.assertEqual(False, self.sended)

        post.publish_at = datetime.today()
        post.save()
        self.assertEqual(True, self.sended)



'''
Inherit your own models from GeneralPost class
and define there application specific fieldd.
'''

import logging
from datetime import datetime

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse, NoReverseMatch

import settings
import signals
from pdb import set_trace

if settings.HAS_TAGGING:
    from tagging.fields import TagField
    from tagging.models import Tag


from utils import upcast

def published(queryset):
    'Filter out not published posts.'
    return queryset.filter(publish_at__lte=datetime.now())

class GeneralPost(models.Model):
    '''Base class for blog posts.'''
    # templates
    list_template             = 'django_dzenlog/list.html'
    detail_template           = 'django_dzenlog/detail.html'
    tagcloud_template         = 'django_dzenlog/tagcloud.html'
    body_detail_template      = 'django_dzenlog/body_detail.html'
    body_list_template        = 'django_dzenlog/body_list.html'
    feed_title_template       = 'django_dzenlog/feed_title.html'
    feed_description_template = 'django_dzenlog/feed_description.html'

    # fields
    author      = models.ForeignKey(User, verbose_name = _('Author'))
    title       = models.CharField(_('Title'), max_length=100)
    slug        = models.SlugField(_('Slug title'), max_length=100, unique=True)
    created_at  = models.DateTimeField(_('Create date'),
                                       blank=True, editable=False)
    updated_at  = models.DateTimeField(_('Update date'),
                                       blank=True, editable=False)
    publish_at  = models.DateTimeField(_('Publish date'), blank=True, null=True)
    comments_on = models.BooleanField(_('Comments On'), default=True)

    if settings.HAS_TAGGING:
        tags    = TagField(_('Tags'))

        def _save_tags(self, *args, **kwargs):
            '''Helper method which makes sure,
               that tags are binded to GeneralPost
               content type.'''
            instance = self.downcast()
            setattr(instance, '_tags_cache', self._tags_cache)
            self._meta.get_field('tags')._save(instance=instance)

        def get_tags(self, *args, **kwargs):
            return Tag.objects.get_for_object(self.downcast())
    else:
        def __init__(self, *args, **kwargs):
            if kwargs.pop('tags', None) is not None:
                logging.getLogger('django_dzenlog').warning(
                    'Tagging is not supported, '
                    'add "tagging" into the INSTALLED_APPS.')
            super(GeneralPost, self).__init__(*args, **kwargs)

    def __unicode__(self):
        return self.title

    def downcast(self):
        return getattr(self, 'generalpost_ptr', self)

    class Meta:
        verbose_name = _('Post')
        verbose_name_plural = _('Posts')
        ordering = ('-publish_at',)

    def get_edit_url(self):
        return reverse('admin-root',
                        args = ['/'.join((
                            self._meta.app_label,
                            self._meta.module_name,
                            unicode(self.id)))])

    def upcast(self):
        return upcast(self)

    def get_absolute_url(self):
        try:
            obj = self.upcast()
            return reverse('dzenlog-%s-details' % obj._meta.module_name,
                            kwargs=dict(slug=self.slug))
        except NoReverseMatch:
            return reverse('dzenlog-%s-details' % \
                                self.downcast()._meta.module_name,
                            kwargs=dict(slug=self.slug))

    def save(self):
        today = datetime.today()
        published = False

        if self.id:
            prev = GeneralPost.objects.get(pk = self.id)
            published = \
                prev.publish_at is None and self.publish_at is not None
        else:
            published = (self.publish_at is not None)
            self.created_at = today

        self.updated_at = today


        result = super(GeneralPost, self).save()

        if settings.HAS_TAGGING:
            self._save_tags()

        if published:
            signals.published.send(sender = type(self), instance = self)
            signals.published.send(sender = GeneralPost, instance = self)
        return result




from setuptools import setup, find_packages

setup(
    name = 'django-dzenlog',
    version = __import__('django_dzenlog').__version__,
    description = '''Django Dzenlog is a set of models and templates, which can be '''
                  '''used to create blogs with different kinds media.''',
    long_description = '''
Django Dzenlog is a set of models and templates, which can be
used to create blogs with different kinds media.

Dzenlog relies on new django's feature -- model inheritance,
so you can derive you own models from dzenlog's models and
add an actual information.

This is very effective way of the code reuse, because dzenlog
will take care about all publishing options, and all what you
need is to describe details, specific to you particular blog.

For example, for can create a blog with two post types: textual
posts and links to internet resources. In that case, all you need
is to define two models: `TextPost` and `LinkPost`. Each of these
models should be derived from `django_dzenlog.models.GeneralPost`.

Features
========

* Simple way to add new types of posts.
* All post types can be agregated in one feed.
* Separate feed for each post type.
* Example projects, which uses most features of this application.
* Tagging support.
    ''',
    keywords = 'django apps blogging',
    license = 'New BSD License',
    author = 'Alexander Artemenko',
    author_email = 'svetlyak.40wt@gmail.com',
    url = 'http://github.com/svetlyak40wt/django-dzenlog/',
    install_requires = [],
    extras_require = {
        'tagging': ['tagging>=0.3-pre'],
    },
    dependency_links = ['http://pypi.aartemenko.com', ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Plugins',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages = find_packages(exclude=['example*']),
    package_data = {
        'templates': ['*.html'],
    },
    include_package_data = True,
)

